==> MeatAxe-2.3.0.tar.gz: src/tuc.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: $
   Comment:     This program uncondenses one or more vectors in (M x N)e.
   --------------------------------------------------------------------------
   Written by Michael Ringe.
   (C) Copyright 1998  Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

/* --------------------------------------------------------------------------
   Global data
   -------------------------------------------------------------------------- */

char TkiName[LAT_MAXBASENAME];	    /* Base name for .tki file */
TK_Info TkInfo;			    /* Data from .tki file */
Lat_Info InfoM, InfoN;		    /* Data from .cfinfo files */
matrix_t *CondMat;		    /* Condensed vectors (read from file) */
matrix_t *UncondMat;		    /* Uncondensed vectors (calculated here) */
char UncondName[LAT_MAXBASENAME];   /* Name of output file */
long DimM, DimN;		    /* Dimensions of M and N */
matrix_t *QMat[LAT_MAXCF];	    /* Q matrices */

static char *helptext[] = {
"SYNTAX",
"    tuc <M> <N> <info> <cond> <uncond>",
"",
"OPTIONS",
"",
"FILES",
"    <M>.cfinfo           i  Info file for left factor",
"    <N>.cfinfo           i  Info file for right factor",
"    <info>.tki           i  Tensor condense info file",
"    <cond>               i  Matrix with condensed vectors",
"    <uncond>             o  Matrix with uncondensed vectors",
NULL};

static proginfo_t pinfo =
   { "tuc", "Tensor Uncondense", "$Revision: $", helptext };




/* --------------------------------------------------------------------------
   AllocateResult() - Calculate dimension and allocate result matrix

   Description:
     This function cacluates the dimensino of M x N, and allocates the
     matrix <UncondMat> which holds the uncondensed vectors.
   -------------------------------------------------------------------------- */

static void AllocateResult()
{
    int i;

    /* Calculate the dimension of M
       ---------------------------- */
    DimN = 0;
    for (i = 0; i < InfoM.NCf; ++i)
	DimM += InfoM.Cf[i].dim * InfoM.Cf[i].mult;

    /* Calculate the dimension of N 
       ---------------------------- */
    DimN = 0;
    for (i = 0; i < InfoN.NCf; ++i)
	DimN += InfoN.Cf[i].dim * InfoN.Cf[i].mult;

    UncondMat = MatAlloc(CondMat->fl,CondMat->nor,DimM * DimN);
}




/* --------------------------------------------------------------------------
   ReadQMatrices() - Read the Q matrices
   -------------------------------------------------------------------------- */

static void ReadQMatrices()
{
    int i;
    char fn[200];

    for (i = 0; i < TkInfo.NCf; ++i)
    {
	sprintf(fn,"%s.q.%d",TkiName,i+1);
	MESSAGE(1,("Reading %s\n",fn));
	QMat[i] = MatLoad(fn);
    }
}


/* --------------------------------------------------------------------------
   Init() - Initialize everything

   Description:
     This function initializes global variables, processes command line
     options and arguments, and reads the .cfinfo and .tki files.

   Arguments:
     <argc>: Number of arguments
     <argv>: Command line arguments
   -------------------------------------------------------------------------- */

static void Init(int argc, char *argv[])

{
    int i;

    /* Initialize the MeatAxe library
       ------------------------------ */
    MtxInit();

    /* Process command line options
       ---------------------------- */
    MtxSetCommandLine(argc,argv,&pinfo);
    while ((i = MtxGetOpt("")) != OPT_END)
    {
	switch (i)
	{
	    default:
		MtxErrorExit("Unknown option '%c'",i);
		break;
	}
    }

    /* Process remaining command line arguments and read input files
       ------------------------------------------------------------- */
    if (opt_ind != argc - 5)
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    Lat_ReadInfo(&InfoM,argv[opt_ind + 0]);
    Lat_ReadInfo(&InfoN,argv[opt_ind + 1]);
    strcpy(TkiName,argv[opt_ind + 2]);
    TK_ReadInfo(&TkInfo,TkiName);
    CondMat = MatLoad(argv[opt_ind + 3]);
    strcpy(UncondName,argv[opt_ind + 4]);
    AllocateResult();
    ReadQMatrices();
}


/* --------------------------------------------------------------------------
   CalculatePositions() - Calculate basis vector indexes

   Description:
     This function calculates the index of the first basis vector that 
     belongs to the tensor product of two given constituents. The 
     constituents are given by their number <cf> (as defined in the tkinfo
     file), and by a number for both M and N specifying which copy of the
     constituent is meant.

   Arguments:
     <cf>: Constituent index in Tki-File (0-based).
     <num_m>: Used to distiguish the different copies, if the constituent
       appears in M with multiplicity > 1. (0-based)
     <num_n>: Used to distiguish the different copies, if the constituent
       appears in N with multiplicity > 1. (0-based)
     <condpos>: Index of the first basis vector in the condensed module.
     <uncondpos>: Index of the first basis vector in the uncondensed module.
   -------------------------------------------------------------------------- */

static void CalculatePositions(int cf, int num_m, int num_n, int *condpos,
    int *uncondpos)

{
    int cfm = TkInfo.CfIndex[0][cf];
    int cfn = TkInfo.CfIndex[1][cf];
    int startm = 0, startn = 0;
    int i;
    MTXCONTEXT(CalculatePositions);

    MTX_ASSERT(cfm >= 0 && cfm <= InfoM.NCf);
    MTX_ASSERT(cfn >= 0 && cfn <= InfoN.NCf);
    MTX_ASSERT(InfoM.Cf[cf].dim == InfoN.Cf[cf].dim);


    /* Calculate starting position in M x N.
       ------------------------------------- */
    for (i = 0; i < cfm; ++i)
	startm += InfoM.Cf[i].dim * InfoM.Cf[i].mult;
    startm += InfoM.Cf[cfm].dim * num_m;
    for (i = 0; i < cfn; ++i)
	startn += InfoN.Cf[i].dim * InfoN.Cf[i].mult;
    startn += InfoN.Cf[cfn].dim * num_n;
    *uncondpos = startm * DimN + startn + 1;

    /* Calculate starting position in (M x N)e.
       ---------------------------------------- */
    *condpos = 1;
    for (i = 0; i < cf; ++i)
    {
	int m = TkInfo.CfIndex[0][i];
	int n = TkInfo.CfIndex[1][i];
	*condpos += InfoM.Cf[m].mult * InfoN.Cf[n].mult * InfoM.Cf[m].spl;
    }
    *condpos += (num_m * InfoN.Cf[cfn].mult +num_n)* InfoM.Cf[cfm].spl;
}


/* --------------------------------------------------------------------------
   UncondenseCf() - Uncondense one component of a vector

   Description:

   Arguments:
     <row>: Number of vector to uncondense (1-based).
     <cf>: Constituent Index in Tki-File (0-based).
   -------------------------------------------------------------------------- */

static void UncondenseCf(int row, int cf)
{
    int i;
    int cfm = TkInfo.CfIndex[0][cf];
    int cfn = TkInfo.CfIndex[1][cf];
    MTXCONTEXT(UncondenseCf);

    for (i = 0; i < InfoM.Cf[cfm].mult; ++i)
    {
	int j;
	for (j = 0; j < InfoN.Cf[cfn].mult; ++j)
	{
	    int k;
	    matrix_t *condvec;
	    int condpos, uncondpos;

	    CalculatePositions(cf,i,j,&condpos,&uncondpos);
	    condvec = MatCut(CondMat,row,condpos,1,QMat[cf]->nor);
	    MatMul(condvec,QMat[cf]);
	    for (k = 1; k <= InfoM.Cf[cfm].dim; ++k)
	    {
		MatCopyRegion(UncondMat,row,uncondpos,condvec,1,1,1,
		    InfoN.Cf[cfn].dim);
		uncondpos += DimN;
	    }
	    MatFree(condvec);
	}
    }
}




/* --------------------------------------------------------------------------
   Uncondense() - Uncondense one vector

   Description:

   Arguments:
     <row>: Number of row to uncondense (starting with 1).
   -------------------------------------------------------------------------- */

static void Uncondense(int row)
{
    int cf;

    /* Loop through all constituents
       ----------------------------- */
    for (cf = 0; cf < TkInfo.NCf; ++cf)
	UncondenseCf(row,cf);
}





/* --------------------------------------------------------------------------
   main() - Program entry point
   -------------------------------------------------------------------------- */

int main(int argc, char *argv[])

{
    int row;

    Init(argc,argv);
    for (row = 1; row <= CondMat->nor; ++row)
	Uncondense(row);
    MatSave(UncondMat,UncondName);
    MtxCleanup();
    return 0;
}

==> MeatAxe-2.3.1.tar.gz: src/tuc.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: $
   Comment:     This program uncondenses one or more vectors in (M x N)e.
   --------------------------------------------------------------------------
   Written by Michael Ringe.
   (C) Copyright 1998  Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

/* --------------------------------------------------------------------------
   Global data
   -------------------------------------------------------------------------- */

char TkiName[LAT_MAXBASENAME];	    /* Base name for .tki file */
TK_Info TkInfo;			    /* Data from .tki file */
Lat_Info InfoM, InfoN;		    /* Data from .cfinfo files */
matrix_t *CondMat;		    /* Condensed vectors (read from file) */
matrix_t *UncondMat;		    /* Uncondensed vectors (calculated here) */
char UncondName[LAT_MAXBASENAME];   /* Name of output file */
long DimM, DimN;		    /* Dimensions of M and N */
matrix_t *QMat[LAT_MAXCF];	    /* Q matrices */

static char *helptext[] = {
"SYNTAX",
"    tuc <M> <N> <info> <cond> <uncond>",
"",
"OPTIONS",
"",
"FILES",
"    <M>.cfinfo           i  Info file for left factor",
"    <N>.cfinfo           i  Info file for right factor",
"    <info>.tki           i  Tensor condense info file",
"    <cond>               i  Matrix with condensed vectors",
"    <uncond>             o  Matrix with uncondensed vectors",
NULL};

static proginfo_t pinfo =
   { "tuc", "Tensor Uncondense", "$Revision: $", helptext };




/* --------------------------------------------------------------------------
   AllocateResult() - Calculate dimension and allocate result matrix

   Description:
     This function cacluates the dimensino of M x N, and allocates the
     matrix <UncondMat> which holds the uncondensed vectors.
   -------------------------------------------------------------------------- */

static void AllocateResult()
{
    int i;

    /* Calculate the dimension of M
       ---------------------------- */
    DimN = 0;
    for (i = 0; i < InfoM.NCf; ++i)
	DimM += InfoM.Cf[i].dim * InfoM.Cf[i].mult;

    /* Calculate the dimension of N 
       ---------------------------- */
    DimN = 0;
    for (i = 0; i < InfoN.NCf; ++i)
	DimN += InfoN.Cf[i].dim * InfoN.Cf[i].mult;

    UncondMat = MatAlloc(CondMat->fl,CondMat->nor,DimM * DimN);
}




/* --------------------------------------------------------------------------
   ReadQMatrices() - Read the Q matrices
   -------------------------------------------------------------------------- */

static void ReadQMatrices()
{
    int i;
    char fn[200];

    for (i = 0; i < TkInfo.NCf; ++i)
    {
	sprintf(fn,"%s.q.%d",TkiName,i+1);
	MESSAGE(1,("Reading %s\n",fn));
	QMat[i] = MatLoad(fn);
    }
}


/* --------------------------------------------------------------------------
   Init() - Initialize everything

   Description:
     This function initializes global variables, processes command line
     options and arguments, and reads the .cfinfo and .tki files.

   Arguments:
     <argc>: Number of arguments
     <argv>: Command line arguments
   -------------------------------------------------------------------------- */

static void Init(int argc, char *argv[])

{
    int i;

    /* Initialize the MeatAxe library
       ------------------------------ */
    MtxInit();

    /* Process command line options
       ---------------------------- */
    MtxSetCommandLine(argc,argv,&pinfo);
    while ((i = MtxGetOpt("")) != OPT_END)
    {
	switch (i)
	{
	    default:
		MtxErrorExit("Unknown option '%c'",i);
		break;
	}
    }

    /* Process remaining command line arguments and read input files
       ------------------------------------------------------------- */
    if (opt_ind != argc - 5)
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    Lat_ReadInfo(&InfoM,argv[opt_ind + 0]);
    Lat_ReadInfo(&InfoN,argv[opt_ind + 1]);
    strcpy(TkiName,argv[opt_ind + 2]);
    TK_ReadInfo(&TkInfo,TkiName);
    CondMat = MatLoad(argv[opt_ind + 3]);
    strcpy(UncondName,argv[opt_ind + 4]);
    AllocateResult();
    ReadQMatrices();
}


/* --------------------------------------------------------------------------
   CalculatePositions() - Calculate basis vector indexes

   Description:
     This function calculates the index of the first basis vector that 
     belongs to the tensor product of two given constituents. The 
     constituents are given by their number <cf> (as defined in the tkinfo
     file), and by a number for both M and N specifying which copy of the
     constituent is meant.

   Arguments:
     <cf>: Constituent index in Tki-File (0-based).
     <num_m>: Used to distiguish the different copies, if the constituent
       appears in M with multiplicity > 1. (0-based)
     <num_n>: Used to distiguish the different copies, if the constituent
       appears in N with multiplicity > 1. (0-based)
     <condpos>: Index of the first basis vector in the condensed module.
     <uncondpos>: Index of the first basis vector in the uncondensed module.
   -------------------------------------------------------------------------- */

static void CalculatePositions(int cf, int num_m, int num_n, int *condpos,
    int *uncondpos)

{
    int cfm = TkInfo.CfIndex[0][cf];
    int cfn = TkInfo.CfIndex[1][cf];
    int startm = 0, startn = 0;
    int i;
    MTXCONTEXT(CalculatePositions);

    MTX_VERIFY(cfm >= 0 && cfm <= InfoM.NCf);
    MTX_VERIFY(cfn >= 0 && cfn <= InfoN.NCf);
    MTX_VERIFY(InfoM.Cf[cf].dim == InfoN.Cf[cf].dim);


    /* Calculate starting position in M x N.
       ------------------------------------- */
    for (i = 0; i < cfm; ++i)
	startm += InfoM.Cf[i].dim * InfoM.Cf[i].mult;
    startm += InfoM.Cf[cfm].dim * num_m;
    for (i = 0; i < cfn; ++i)
	startn += InfoN.Cf[i].dim * InfoN.Cf[i].mult;
    startn += InfoN.Cf[cfn].dim * num_n;
    *uncondpos = startm * DimN + startn + 1;

    /* Calculate starting position in (M x N)e.
       ---------------------------------------- */
    *condpos = 1;
    for (i = 0; i < cf; ++i)
    {
	int m = TkInfo.CfIndex[0][i];
	int n = TkInfo.CfIndex[1][i];
	*condpos += InfoM.Cf[m].mult * InfoN.Cf[n].mult * InfoM.Cf[m].spl;
    }
    *condpos += (num_m * InfoN.Cf[cfn].mult +num_n)* InfoM.Cf[cfm].spl;
}


/* --------------------------------------------------------------------------
   UncondenseCf() - Uncondense one component of a vector

   Description:

   Arguments:
     <row>: Number of vector to uncondense (1-based).
     <cf>: Constituent Index in Tki-File (0-based).
   -------------------------------------------------------------------------- */

static void UncondenseCf(int row, int cf)
{
    int i;
    int cfm = TkInfo.CfIndex[0][cf];
    int cfn = TkInfo.CfIndex[1][cf];
    MTXCONTEXT(UncondenseCf);

    MTX_VERIFY(row > 0);
    for (i = 0; i < InfoM.Cf[cfm].mult; ++i)
    {
	int j;
	for (j = 0; j < InfoN.Cf[cfn].mult; ++j)
	{
	    int k;
	    matrix_t *condvec;
	    int condpos, uncondpos;

	    CalculatePositions(cf,i,j,&condpos,&uncondpos);
	    condvec = MatCut(CondMat,row,condpos,1,QMat[cf]->nor);
	    MatMul(condvec,QMat[cf]);
	    for (k = 1; k <= InfoM.Cf[cfm].dim; ++k)
	    {
		MatCopyRegion(UncondMat,row,uncondpos,condvec,1,1,1,
		    InfoN.Cf[cfn].dim);
		uncondpos += DimN;
	    }
	    MatFree(condvec);
	}
    }
}




/* --------------------------------------------------------------------------
   Uncondense() - Uncondense one vector

   Description:

   Arguments:
     <row>: Number of row to uncondense (starting with 1).
   -------------------------------------------------------------------------- */

static void Uncondense(int row)
{
    int cf;

    /* Loop through all constituents
       ----------------------------- */
    for (cf = 0; cf < TkInfo.NCf; ++cf)
	UncondenseCf(row,cf);
}





/* --------------------------------------------------------------------------
   main() - Program entry point
   -------------------------------------------------------------------------- */

int main(int argc, char *argv[])

{
    int row;

    Init(argc,argv);
    for (row = 1; row <= CondMat->nor; ++row)
	Uncondense(row);
    MatSave(UncondMat,UncondName);
    MtxCleanup();
    return 0;
}
